
__all__ = ['Analyzer']


class Analyzer:
    """
    Analyzers are run against images to update the information we hold about
    them (this information is added to their meta data).
    """

    def __init__(self, name, **kwargs):

        # The name of the analyzer
        self._name = name

        # The arguments for the analyzer
        self._args = {k: v for k, v in kwargs.items() if v is not None}

    def to_json_type(self):
        return [self._name, self._args]


from . import images




from . import Transform

__all__ = [
    'AutoOrient',
    'Crop',
    'Fit',
    'FocalPointCrop',
    'Output',
    'Rotate',
    'SingleFrame'
]


class AutoOrient(Transform):
    """
    Auto orient an image based on its Exif (Exchangeable image file format)
    data.
    """

    def __init__(self):
        super().__init__('auto_orient')


class Crop(Transform):
    """
    Crop an image.
    """

    def __init__(self, top, left, bottom, right):
        super().__init__(
            'crop',
            top=top,
            left=left,
            bottom=bottom,
            right=right
        )


class Fit(Transform):
    """
    Fit an image to a width/height.
    """

    def __init__(self, width, height, resample=None):
        super().__init__(
            'fit',
            width=width,
            height=height,
            resample=resample
        )


class FocalPointCrop(Transform):
    """
    Crop an image using around it's focal point (if no focal point is defined
    for the image the entire image will be considered the focal point).
    """

    def __init__(
        self,
        aspect_ratio=None,
        padding_top=None,
        padding_left=None,
        padding_bottom=None,
        padding_right=None
    ):
        super().__init__(
            'focal_point_crop',
            aspect_ratio=aspect_ratio,
            padding_top=padding_top,
            padding_left=padding_left,
            padding_bottom=padding_bottom,
            padding_right=padding_right
        )


class Output(Transform):
    """
    Output an image variation.
    """

    def __init__(
        self,
        image_format,
        quality=None,
        lossless=None,
        progressive=None,
        versioned=None
    ):
        super().__init__(
            'output',
            image_format=image_format,
            quality=quality,
            lossless=lossless,
            progressive=progressive,
            versioned=versioned
        )


class Rotate(Transform):
    """
    Rotate an image.
    """

    def __init__(self, degrees):
        super().__init__(
            'rotate',
            degrees=degrees
        )


class SingleFrame(Transform):
    """
    Extract a single frame from an animated image.
    """

    def __init__(self, frame_number=None):
        super().__init__(
            'single_frame',
            frame_number=frame_number
        )



import io
import json

import requests

from . import exceptions

__all__ = ['Client']


class Client:
    """
    A client for the H51 (Hangar51) API.
    """

    def __init__(
        self,
        api_key,
        api_base_url='https://api.h51.io',
        timeout=None
    ):

        # A key used to authenticate API calls to an account
        self._api_key = api_key

        # The base URL to use when calling the API
        self._api_base_url = api_base_url

        # The period of time before requests to the API should timeout
        self._timeout = timeout

        # NOTE: Rate limiting information is only available after a request
        # has been made.

        # The maximum number of requests per second that can be made with the
        # given API key.
        self._rate_limit = None

        # The time (seconds since epoch) when the current rate limit will
        # reset.
        self._rate_limit_reset = None

        # The number of requests remaining within the current limit before the
        # next reset.
        self._rate_limit_remaining = None

    @property
    def rate_limit(self):
        return self._rate_limit

    @property
    def rate_limit_reset(self):
        return self._rate_limit_reset

    @property
    def rate_limit_remaining(self):
        return self._rate_limit_remaining

    def __call__(self,
        method,
        path,
        params=None,
        data=None,
        files=None,
        download=False
    ):
        """Call the API"""

        # Build headers
        headers = {'X-H51-APIKey': self._api_key}

        if not download:
            headers['Accept'] = 'application/json'

        if params:
            # Filter out parameters set to `None`
            params = {k: v for k, v in params.items() if v is not None}

        if data:
            # Filter out data set to `None`
            data = {k: v for k, v in data.items() if v is not None}

        # Make the request
        r = getattr(requests, method.lower())(
            f'{self._api_base_url}/{path}',
            headers=headers,
            params=params,
            data=data,
            files=files,
            timeout=self._timeout
        )

        # Update the rate limit
        if 'X-H51-RateLimit-Limit' in r.headers:
            self._rate_limit = int(r.headers['X-H51-RateLimit-Limit'])
            self._rate_limit_reset \
                    = float(r.headers['X-H51-RateLimit-Reset'])
            self._rate_limit_remaining \
                    = int(r.headers['X-H51-RateLimit-Remaining'])

        # Handle a successful response
        if r.status_code in [200, 204]:

            if download:
                return io.BytesIO(r.content)

            if r.headers.get('Content-Type', '')\
                    .startswith('application/json'):

                return r.json()

            return None

        # Raise an error related to the response
        try:
            error = r.json()

        except ValueError:
            error = {}

        error_cls = exceptions.H51Exception.get_class_by_status_code(
            r.status_code
        )

        raise error_cls(
            r.status_code,
            error.get('hint'),
            error.get('arg_errors')
        )
